/**
 * @file
 * Toggles the 'dark' class based on the selected dark mode.
 */

((Drupal, once) => {
  /**
   * Attaches behavior to toggle a class based on the selected dark mode.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Toggles the 'dark' class on the HTML element based on the selected dark
   *   mode, and updates the 'data-dark-mode-source' attribute accordingly.
   */
  Drupal.behaviors.darkModeToggle = {
    attach(context) {
      once('darkModeToggle', 'html', context).forEach(() => {
        context
          .querySelector('.dark-mode-toggle__button__light')
          .addEventListener('click', () => {
            // When the user explicitly chooses the light mode.
            localStorage.setItem('dark-mode', 'light');
            document.documentElement.classList.remove('dark');
            document.documentElement.setAttribute(
              'data-dark-mode-source',
              'user',
            );
          });

        context
          .querySelector('.dark-mode-toggle__button__dark')
          .addEventListener('click', () => {
            // When the user explicitly chooses the dark mode.
            localStorage.setItem('dark-mode', 'dark');
            document.documentElement.classList.add('dark');
            document.documentElement.setAttribute(
              'data-dark-mode-source',
              'user',
            );
          });

        context
          .querySelector('.dark-mode-toggle__button__system')
          .addEventListener('click', () => {
            // Whenever the user explicitly chooses to respect/switch back to the
            // OS preference.
            localStorage.removeItem('dark-mode');
            document.documentElement.classList.toggle(
              'dark',
              window.matchMedia('(prefers-color-scheme: dark)').matches,
            );
            document.documentElement.setAttribute(
              'data-dark-mode-source',
              'system',
            );
          });

        window
          .matchMedia('(prefers-color-scheme: dark)')
          .addEventListener('change', (e) => {
            // When the system preference changes and the user has chosen to
            // respect the system preference.
            if (
              document.documentElement.getAttribute('data-dark-mode-source') ===
              'system'
            ) {
              document.documentElement.classList.toggle('dark', e.matches);
            }
          });
      });
    },
  };
})(Drupal, once);
